package Plugins::DRS::Plugin;

use strict;

use Slim::Networking::SimpleAsyncHTTP;
use Slim::Utils::Log;

use constant SEARCH_URL => 'http://www.herger.net/cgi-bin/getartwork.fcgi';

my $log = Slim::Utils::Log->addLogCategory({
	'category'     => 'plugin.drs',
	'description'  => 'PLUGIN_DRS_META',
});

# http://www.broadcast.ch/audiostreams.aspx
my $icons = {
	drs1 => 'https://www.herger.net/logos/radio-srf-1.png',
	drs2 => 'https://www.herger.net/logos/radio-srf-2.png',
	drs3 => 'https://www.herger.net/logos/radio-srf-3.png',
	drs4 => 'https://www.herger.net/logos/radio-srf-4.png',
	drsmw    => 'https://www.herger.net/logos/radio-srf-musikwelle.png',
	drsvirus => 'https://www.herger.net/logos/radio-srf-virus.png',
	rsp  => 'https://www.herger.net/logos/radio-swiss-pop.png',
	rsj  => 'https://www.herger.net/logos/radio-swiss-jazz.png',
	rsc_de  => 'https://www.herger.net/logos/radio-swiss-classic.png',
	'1ere'  => 'https://www.herger.net/logos/premiere.png',
	espace  => 'https://www.herger.net/logos/espace.png',
	couleur => 'https://www.herger.net/logos/couleur3.png',
	reteuno => 'https://www.herger.net/logos/rete-uno.png',
	retedue => 'https://www.herger.net/logos/rete-due.png',
	retetre => 'https://www.herger.net/logos/rete-tre.png',
	kanalk  => 'http://cdn-radiotime-logos.tunein.com/s2993q.png',
};

$icons->{rsc} = $icons->{rsc_de};

my $cantArtwork = {
	drs4 => 1,
};

my $urlParseRegex = qr/(?:srg-ssr|streamserver|skypro|swisstxt|stream).*(drs(?:[1-4]|virus|mw)|rete(?:uno|due|tre)|regi_[a-z_]+|drs_event|\b(?:rsp|1ere|espace|rsj|rsc_de)\b|\bcouleur|kanalk)/i;

my ($imageProxy, $lastMaxSize);

sub initPlugin {
	my $class = shift;

	Slim::Formats::RemoteMetadata->registerParser(
		match => $urlParseRegex,
		func  => \&parser,
	);

	# support for new LMS 7.8 local image proxy
	# register custom image proxy parser to optimize artwork download
	eval {
		require Slim::Web::ImageProxy;

		$imageProxy = sub {
			my ($url, $spec) = @_;

			main::DEBUGLOG && $log->debug("Radio SRF artwork - let's get the smallest version fitting our needs: $url for $spec");

			my ($url2, $sizes) = $url =~ /(.*)\?sizes=(.*)$/;

			return $url unless $url2 && $sizes;

			my %sizeMap = map {
				/(\d+).*/ && $1 => $_
			} grep {
				# let's not bother with the really small files, though they might be quicker to download
				/(\d+)/ && $1 >= 300
			} split(/\|/, $sizes);

			# not reported as available, but 600x600 seems to be there
			$sizeMap{600} = '600x600';

			my $size = Slim::Web::ImageProxy->getRightSize($spec, \%sizeMap) || '_';

			if ($size eq '_') {
				# optimization for Radio: often it would require the full size image, which all too often is several megabytes
				if ($spec =~ /^320x/) {
					$size = $sizeMap{300} || '_';
				}
				else {
					$lastMaxSize = time() + 600;
				}
			}
			elsif ($lastMaxSize && $lastMaxSize < time()) {
				$size = '_';
			}

			$url2 =~ s/(\/i\/u\/)(?:_\/)?/$1$size\//;
			$url2 =~ s/\.png(\?)?/.jpg$1/;

			main::DEBUGLOG && $log->debug("Going to get $url2 for $spec");

			return $url2;
		};

		Slim::Web::ImageProxy->registerHandler(
			match => qr/(?:la?st\.fm|lastfm\.freetls).*\?sizes=[\d\|sx]+$/,
			func  => $imageProxy,
		);

		main::DEBUGLOG && $log->debug("Successfully registered image proxy for SRF artwork");
	}
}


sub parser {
	my ( $client, $url, $metadata ) = @_;

	if ($metadata =~ (/StreamTitle=\'(.*?)\'(?:;|$)/)) {
		my $titleInfo = $1;

		# what station?
		$url =~ $urlParseRegex;
		my $station = lc($1 || 'drs3');

		main::DEBUGLOG && $log->is_debug && $log->debug("Icy metadata received for $station: $titleInfo");

		my ($artist, $title) = split(' - ', $titleInfo);

		my $info = {
			icon   => $icons->{$station},
			cover  => $icons->{$station},
			artist => undef,
			title  => $titleInfo,
		};

		if ($title && $artist) {

			$info->{artist} = _cleanup($artist);
			$info->{title} = _cleanup($title);

			getArtwork( $client, $station, $info );
		}

		my $controller = $client->controller()->songStreamController();
		my $song = $controller->song() if $controller;

		if ($song) {
			$song->pluginData( wmaMeta => $info );

			Slim::Control::Request::notifyFromArray( $client, [ 'newmetadata' ] );
			Slim::Music::Info::setDelayedTitle( $client, $url, $titleInfo );
		}

		return 1;
	}

	main::DEBUGLOG && $log->is_debug && $log->debug("No metadata set for $url");

	return 0;
}

sub getArtwork {
	my ( $client, $station, $info ) = @_;

	return if $cantArtwork->{$station};

	my $all = $info->{artist} . ' ' . $info->{title};
	return if $all =~ /\s*(?:jetzt|\d\d:\d\d): +/i || $all =~ /srf.*?(?:zambo|1|2|3|4|virus)/i;
	return if !$info->{artist} || $all =~ /(?:\Q$station\E|la 1.re|radio swiss)/i;

	main::DEBUGLOG && $log->is_debug && $log->debug("Let's get some artwork...");

	$info->{title} =~ s/.+(?:20|19)\d\d$//;	# RETE is adding the year or something to every title

	my $args = '?track=' . URI::Escape::uri_escape_utf8( $info->{title} || '' )
		. '&artist=' . URI::Escape::uri_escape_utf8( $info->{artist} || '' );

	my $http = Slim::Networking::SimpleAsyncHTTP->new(
		sub {
			my $html = shift;

			if ( $html && $html->content && $html->content =~ /^http/ ) {
				my $imageUrl = $html->content;
				$imageUrl =~ s/\?sizes=.*$// unless $imageProxy;

				main::DEBUGLOG && $log->is_debug && $log->debug("Found title artwork: " . $imageUrl);

				$client->playingSong->pluginData( wmaMeta => {
					icon   => $imageUrl,
					cover  => $imageUrl,
					artist => $info->{artist},
					title  => $info->{title},
				} );

				Slim::Control::Request::notifyFromArray( $client, [ 'newmetadata' ] );
			}
			elsif ( $html && $html->content ) {

				main::DEBUGLOG && $log->is_debug && $log->debug("Nothing found: " . $html->content);

			}
			else {

				main::DEBUGLOG && $log->is_debug && $log->debug(Data::Dump::dump($html));

			}
		},
		sub {
			logError(Data::Dump::dump(shift));
		},
	);

	$http->get( SEARCH_URL . $args );

	return 1;
}

sub _cleanup {
	my $s = shift;

	$s =~ s/ \(CH\)\s*$//;

	utf8::decode($s);

	# only mess with upper/lowercase if we don't have any lower case yet
	return $s if $s =~ /[a-z]+/;

	# uppercase all first characters
	$s =~ s/([\w'\`[:alpha:]äöüéàèçôî]+)/\u\L$1/ig;

	# fallback in case locale is wrong
	$s =~ tr/äöüàéèçôî/ÄÖÜÀÉÈÇÔÎ/;

	$s =~ s/^\s+//;
	$s =~ s/[\:\s]+$//;

	return $s;
}

1;